@@ -111,6 +111,13 @@ void KUrlTest::testSetQuery()
   QCOMPARE( url1.query(), QString("?") );
   url1.setQuery( QString() );
   QCOMPARE( url1.query(), QString() );
+
+  url1.setQuery("4=2+2");
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi?4=2+2"));
+  QCOMPARE( url1.query(), QString("?4=2+2"));
+  url1.setQuery("4=2%2B2");
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi?4=2%2B2")); // %2B should not be decoded to +
+  QCOMPARE( url1.query(), QString("?4=2%2B2"));
 }
 
 void KUrlTest::testEmptyNullReference()
@@ -272,6 +279,10 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QCOMPARE( url0.htmlRef(), QString("/?:@-._~!$&'()*+,;=%A") );
   QCOMPARE( url0.url(), QString("http://www.kde.org#/?:@-._~!$&'()*+,;=%25A") );
 
+  url0 = "http://www.kde.org/foo.cgi?4=2+2";
+  QCOMPARE( url0.url(), QString("http://www.kde.org/foo.cgi?4=2+2") );
+  url0 = "http://www.kde.org/foo.cgi?4=2%2B2";
+  QCOMPARE( url0.url(), QString("http://www.kde.org/foo.cgi?4=2%2B2") );  // %2B should not be decoded to +
 
   KUrl fileURL( "file:///home/dfaure/myfile" );
   QCOMPARE( fileURL.url(), QString("file:///home/dfaure/myfile") );
@@ -657,8 +668,11 @@ void KUrlTest::testUpUrl()
   // Going up from a relative url is not supported (#170695)
   KUrl invalid("tmp");
   QVERIFY(invalid.isValid());
+  QVERIFY(invalid.hasPath());
+  QVERIFY(invalid.directory().isEmpty());
   up = invalid.upUrl();
   QVERIFY(!up.isValid());
+
 }
 
 void KUrlTest::testSetFileName() // and addPath
@@ -866,6 +880,12 @@ void KUrlTest::testIsRelative()
   QVERIFY(KUrl("blah").isRelative());
   QVERIFY(!KUrl("http://www.kde.org").isRelative());
   QVERIFY(KUrl("foo/bar").isRelative());
+  KUrl rel1("f%o#o@b a&r?");
+  QVERIFY(rel1.isRelative());
+  QCOMPARE(rel1.prettyUrl(), QString("f%25o#o@b a&r?"));
+  QCOMPARE(rel1.url(), QString("f%25o#o@b%20a&r?"));
+  QCOMPARE(rel1.path(), QString("f%o"));
+  QCOMPARE(QUrl(rel1).toString(), QString("f%o#o@b a&r?"));
 }
 
 void KUrlTest::testRelativePath()
@@ -1254,6 +1274,8 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   KUrl flashBase("http://www.youtube.com/?v=JvOSnRD5aNk");
   KUrl flashComposed(flashBase, flashRel);
   QCOMPARE(flashComposed.url(), QString("javascript:window.location+%22__flashplugin_unique__%22"));
+
+  QCOMPARE(KUrl(KUrl("data:text/plain, main"), "data:text/plain, subpage").url(), QString("data:text/plain,%20subpage"));
 }
 
 void KUrlTest::testSetEncodedFragment_data()
@@ -1764,6 +1786,10 @@ void KUrlTest::testMailto()
   KUrl url183433("mailto:test[at]gmail[dot]com");
   QCOMPARE(url183433.prettyUrl(), QString("mailto:test[at]gmail[dot]com"));
   QCOMPARE(url183433.url(), QString("mailto:test[at]gmail[dot]com"));
+
+  // Make sure prettyUrl() doesn't convert the url to mailto://...
+  KUrl urlNoPath("mailto:?to=test@example.com");
+  QCOMPARE(urlNoPath.prettyUrl(), QString("mailto:?to=test@example.com"));
 }
 
 void KUrlTest::testSmb()
