@@ -46,7 +46,7 @@ K_GLOBAL_STATIC(KCharSelectData, s_data)
 class KCharSelectTablePrivate
 {
 public:
-    KCharSelectTablePrivate(KCharSelectTable *q): q(q), model()
+    KCharSelectTablePrivate(KCharSelectTable *q): q(q), model(0)
         {}
 
     KCharSelectTable *q;
@@ -58,7 +58,7 @@ public:
 
     void _k_resizeCells();
     void _k_doubleClicked(const QModelIndex & index);
-    void _k_slotCurrentChanged(const QModelIndex & current, const QModelIndex & previous);
+    void _k_slotSelectionChanged(const QItemSelection & selected, const QItemSelection & deselected);
 };
 
 class KCharSelect::KCharSelectPrivate
@@ -73,7 +73,7 @@ public:
 
     enum { MaxHistoryItems = 100 };
 
-    KCharSelectPrivate(KCharSelect *q) 
+    KCharSelectPrivate(KCharSelect *q)
         : q(q)
           ,searchLine(0)
           ,searchMode(false)
@@ -180,7 +180,8 @@ void KCharSelectTable::setChar(const QChar &c)
 {
     int pos = d->chars.indexOf(c);
     if (pos != -1) {
-        setCurrentIndex(model()->index(pos / model()->columnCount(), pos % model()->columnCount()));
+        const int columnCount = model()->columnCount();
+        setCurrentIndex(model()->index(pos / columnCount, pos % columnCount));
     }
 }
 
@@ -196,7 +197,7 @@ void KCharSelectTable::setContents(QList<QChar> chars)
     setSelectionModel(selectionModel);
     setSelectionBehavior(QAbstractItemView::SelectItems);
     setSelectionMode(QAbstractItemView::SingleSelection);
-    connect(selectionModel, SIGNAL(currentChanged(const QModelIndex & , const QModelIndex &)), this, SLOT(_k_slotCurrentChanged(const QModelIndex &, const QModelIndex &)));
+    connect(selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(_k_slotSelectionChanged(QItemSelection,QItemSelection)));
     connect(d->model, SIGNAL(showCharRequested(QChar)), this, SIGNAL(showCharRequested(QChar)));
     delete m; // this should hopefully delete aold selection models too, since it is the parent of them (didn't track, if there are setParent calls somewhere. Check that (jowenn)
 }
@@ -211,11 +212,12 @@ void KCharSelectTable::scrollTo(const QModelIndex & index, ScrollHint hint)
     }
 }
 
-void KCharSelectTablePrivate::_k_slotCurrentChanged(const QModelIndex & current, const QModelIndex & previous)
+void KCharSelectTablePrivate::_k_slotSelectionChanged(const QItemSelection & selected, const QItemSelection & deselected)
 {
-    Q_UNUSED(previous);
-    if (!model) return;
-    QVariant temp = model->data(current, KCharSelectItemModel::CharacterRole);
+    Q_UNUSED(deselected);
+    if (!model || selected.indexes().isEmpty())
+        return;
+    QVariant temp = model->data(selected.indexes().at(0), KCharSelectItemModel::CharacterRole);
     if (temp.type() != QVariant::Char)
         return;
     QChar c = temp.toChar();
@@ -233,33 +235,58 @@ void KCharSelectTable::resizeEvent(QResizeEvent * e)
 
 void KCharSelectTablePrivate::_k_resizeCells()
 {
-    if (!q->model()) return;
-    static_cast<KCharSelectItemModel*>(q->model())->updateColumnCount(q->viewport()->size().width());
+    KCharSelectItemModel *model = static_cast<KCharSelectItemModel*>(q->model());
+    if (!model) return;
 
-    QChar oldChar = q->chr();
+    const int viewportWidth = q->viewport()->size().width();
 
-    const int new_w   = q->viewport()->size().width() / q->model()->columnCount(QModelIndex());
-    const int columns = q->model()->columnCount(QModelIndex());
-    const int rows = q->model()->rowCount(QModelIndex());
+    QFontMetrics fontMetrics(font);
+
+    // Determine the max width of the displayed characters
+    // fontMetrics.maxWidth() doesn't help because of font fallbacks
+    // (testcase: Malayalam characters)
+    int maxCharWidth = 0;
+    const QList<QChar> chars = model->chars();
+    for (int i = 0 ; i < chars.size(); ++i) {
+        maxCharWidth = qMax(maxCharWidth, fontMetrics.width(chars.at(i)));
+    }
+    // Avoid too narrow cells
+    maxCharWidth = qMax(maxCharWidth, 2 * fontMetrics.xHeight());
+    maxCharWidth = qMax(maxCharWidth, fontMetrics.height());
+    // Add the necessary padding, trying to match the delegate
+    const int textMargin = q->style()->pixelMetric(QStyle::PM_FocusFrameHMargin, 0, q) + 1;
+    maxCharWidth += 2 * textMargin;
+
+    const int columns = qMax(1, viewportWidth / maxCharWidth);
+    model->setColumnCount(columns);
+
+    const QChar oldChar = q->chr();
+
+    const int new_w = viewportWidth / columns;
+    const int rows = model->rowCount();
     q->setUpdatesEnabled(false);
-    QHeaderView* hv = q->horizontalHeader();
-    int spaceLeft = q->viewport()->size().width() % new_w + 1;
-    for (int i = 0;i <= columns;i++) {
+    QHeaderView *hHeader = q->horizontalHeader();
+    const int spaceLeft = viewportWidth - new_w * columns;
+    for (int i = 0 ; i <= columns; i++ ) {
         if (i < spaceLeft) {
-            hv->resizeSection(i, new_w + 1);
+            hHeader->resizeSection(i, new_w + 1);
         } else {
-            hv->resizeSection(i, new_w);
+            hHeader->resizeSection(i, new_w);
         }
     }
 
-    hv = q->verticalHeader();
+    QHeaderView *vHeader = q->verticalHeader();
 #ifdef Q_WS_WIN
-    const int new_h = QFontMetrics(font).lineSpacing() + 1;
+    int new_h = fontMetrics.lineSpacing() + 1;
 #else
-    const int new_h = QFontMetrics(font).xHeight() * 3;
+    int new_h = fontMetrics.xHeight() * 3;
 #endif
+    const int fontHeight = fontMetrics.height();
+    if (new_h < 5 || new_h < 4 + fontHeight) {
+        new_h = qMax(5, 4 + fontHeight);
+    }
     for (int i = 0;i < rows;i++) {
-        hv->resizeSection(i, new_h);
+        vHeader->resizeSection(i, new_h);
     }
 
     q->setUpdatesEnabled(true);
@@ -269,7 +296,7 @@ void KCharSelectTablePrivate::_k_resizeCells()
 void KCharSelectTablePrivate::_k_doubleClicked(const QModelIndex & index)
 {
     QChar c = model->data(index, KCharSelectItemModel::CharacterRole).toChar();
-    if (c.isPrint()) {
+    if (s_data->isPrint(c)) {
         emit q->activated(c);
     }
 }
@@ -285,7 +312,7 @@ void KCharSelectTable::keyPressEvent(QKeyEvent *e)
     case Qt::Key_Enter: case Qt::Key_Return: {
             if (!currentIndex().isValid()) return;
             QChar c = d->model->data(currentIndex(), KCharSelectItemModel::CharacterRole).toChar();
-            if (c.isPrint()) {
+            if (s_data->isPrint(c)) {
                 emit activated(c);
             }
         }
@@ -300,11 +327,13 @@ void KCharSelectTable::keyPressEvent(QKeyEvent *e)
 /* Class: KCharSelect                                             */
 /******************************************************************/
 
+#ifndef KDE_NO_DEPRECATED
 KCharSelect::KCharSelect(QWidget *parent, const Controls controls)
         : QWidget(parent), d(new KCharSelectPrivate(this))
 {
     init(controls, NULL);
 }
+#endif
 
 KCharSelect::KCharSelect(
         QWidget *parent
@@ -393,7 +422,7 @@ void KCharSelect::init(const Controls controls, KActionCollection *collection)
     d->fontSizeSpinBox->setSingleStep(1);
     d->fontSizeSpinBox->setToolTip(i18n("Set font size"));
 
-    connect(d->fontCombo, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(_k_fontSelected()));
+    connect(d->fontCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(_k_fontSelected()));
     connect(d->fontSizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(_k_fontSelected()));
 
     if ((HistoryButtons & controls) || (FontCombo & controls) || (FontSize & controls) || (BlockCombos & controls)) {
@@ -436,10 +465,10 @@ void KCharSelect::init(const Controls controls, KActionCollection *collection)
 
     setCurrentFont(QFont());
 
-    connect(d->charTable, SIGNAL(focusItemChanged(const QChar &)), this, SLOT(_k_updateCurrentChar(const QChar &)));
-    connect(d->charTable, SIGNAL(activated(const QChar &)), this, SIGNAL(charSelected(const QChar &)));
-    connect(d->charTable, SIGNAL(focusItemChanged(const QChar &)),
-            this, SIGNAL(currentCharChanged(const QChar &)));
+    connect(d->charTable, SIGNAL(focusItemChanged(QChar)), this, SLOT(_k_updateCurrentChar(QChar)));
+    connect(d->charTable, SIGNAL(activated(QChar)), this, SIGNAL(charSelected(QChar)));
+    connect(d->charTable, SIGNAL(focusItemChanged(QChar)),
+            this, SIGNAL(currentCharChanged(QChar)));
 
     connect(d->charTable, SIGNAL(showCharRequested(QChar)), this, SLOT(setCurrentChar(QChar)));
 
@@ -659,7 +688,7 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(const QChar &c)
         html += "<p style=\"margin-bottom: 0px;\">" + i18n("See also:") + "</p><ul style=\"margin-top: 0px;\">";
         foreach(const QChar &c2, seeAlso) {
             html += "<li><a href=\"" + QString::number(c2.unicode(), 16) + "\">";
-            if (c2.isPrint()) {
+            if (s_data->isPrint(c2)) {
                 html += "&#" + QString::number(c2.unicode()) + "; ";
             }
             html += s_data->formatCode(c2.unicode()) + ' ' + Qt::escape(s_data->name(c2)) + "</a></li>";
@@ -726,7 +755,7 @@ void KCharSelect::KCharSelectPrivate::_k_slotUpdateUnicode(const QChar &c)
 
     html += "<p><b>" + i18n("General Character Properties") + "</b><br>";
     html += i18n("Block: ") + s_data->block(c) + "<br>";
-    html += i18n("Unicode category: ") + s_data->categoryText(c.category()) + "</p>";
+    html += i18n("Unicode category: ") + s_data->categoryText(s_data->category(c)) + "</p>";
 
     QByteArray utf8 = QString(c).toUtf8();
 
@@ -759,7 +788,7 @@ QString KCharSelect::KCharSelectPrivate::createLinks(QString s)
     foreach(const QString &c, chars2) {
         int unicode = c.toInt(0, 16);
         QString link = "<a href=\"" + c + "\">";
-        if (QChar(unicode).isPrint()) {
+        if (s_data->isPrint(QChar(unicode))) {
             link += "&#" + QString::number(unicode) + ";&nbsp;";
         }
         link += "U+" + c + ' ';
@@ -855,7 +884,8 @@ void  KCharSelect::KCharSelectPrivate::_k_linkClicked(QUrl url)
 QVariant KCharSelectItemModel::data(const QModelIndex &index, int role) const
 {
     int pos = m_columns * (index.row()) + index.column();
-    if (pos >= m_chars.size() || index.row() < 0 || index.column() < 0) {
+    if (!index.isValid() || pos < 0 || pos >= m_chars.size()
+            || index.row() < 0 || index.column() < 0) {
         if (role == Qt::BackgroundColorRole) {
             return QVariant(qApp->palette().color(QPalette::Button));
         }
@@ -863,9 +893,7 @@ QVariant KCharSelectItemModel::data(const QModelIndex &index, int role) const
     }
 
     QChar c = m_chars[pos];
-    if (!index.isValid())
-        return QVariant();
-    else if (role == Qt::ToolTipRole) {
+    if (role == Qt::ToolTipRole) {
         QString result = s_data->display(c, m_font) + "<br />" + Qt::escape(s_data->name(c)) + "<br />" +
                          i18n("Unicode code point:") + ' ' + s_data->formatCode(c.unicode()) + "<br />" +
                          i18nc("Character", "In decimal:") + ' ' + QString::number(c.unicode());
@@ -873,12 +901,12 @@ QVariant KCharSelectItemModel::data(const QModelIndex &index, int role) const
     } else if (role == Qt::TextAlignmentRole)
         return QVariant(Qt::AlignHCenter | Qt::AlignVCenter);
     else if (role == Qt::DisplayRole) {
-        if (c.isPrint())
+        if (s_data->isPrint(c))
             return QVariant(c);
         return QVariant();
     } else if (role == Qt::BackgroundColorRole) {
         QFontMetrics fm = QFontMetrics(m_font);
-        if (fm.inFont(c) && c.isPrint())
+        if (fm.inFont(c) && s_data->isPrint(c))
             return QVariant(qApp->palette().color(QPalette::Base));
         else
             return QVariant(qApp->palette().color(QPalette::Button));
@@ -890,5 +918,41 @@ QVariant KCharSelectItemModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
+bool KCharSelectItemModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent)
+{
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
+    if (action == Qt::IgnoreAction) {
+        return true;
+    }
+
+    if (!data->hasText()) {
+        return false;
+    }
+
+    if (column > 0) {
+        return false;
+    }
+    QString text = data->text();
+    if (text.isEmpty()) {
+        return false;
+    }
+    emit showCharRequested(text[0]);
+    return true;
+}
+
+void KCharSelectItemModel::setColumnCount(int columns)
+{
+    emit layoutAboutToBeChanged();
+    m_columns = columns;
+    emit layoutChanged();
+}
+
+QList<QChar> KCharSelectItemModel::chars() const
+{
+    return m_chars;
+}
+
+
 #include "kcharselect.moc"
 #include "kcharselect_p.moc"
