@@ -95,12 +95,14 @@ KNumInput::KNumInput(QWidget* parent)
     setFocusPolicy(Qt::StrongFocus);
 }
 
+#ifndef KDE_NO_DEPRECATED
 KNumInput::KNumInput(QWidget* parent, KNumInput* below)
     : QWidget(parent), d(new KNumInputPrivate(this, below))
 {
     setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
     setFocusPolicy(Qt::StrongFocus);
 }
+#endif
 
 KNumInput::~KNumInput()
 {
@@ -289,10 +291,13 @@ int KIntSpinBox::valueFromText(const QString &text) const
 {
     bool ok;
     QString theText = text;
+    if (theText.startsWith(prefix())) {
+        theText.remove(0, prefix().length());
+    }
     if (theText.endsWith(suffix())) {
         theText.chop(suffix().length());
     }
-    return theText.toInt(&ok, d->val_base);
+    return theText.trimmed().toInt(&ok, d->val_base);
 }
 
 void KIntSpinBox::setEditFocus(bool mark)
@@ -330,12 +335,14 @@ public:
 };
 
 
+#ifndef KDE_NO_DEPRECATED
 KIntNumInput::KIntNumInput(KNumInput* below, int val, QWidget *parent, int _base)
     : KNumInput(parent, below)
     , d(new KIntNumInputPrivate(this, val))
 {
     init(val, _base);
 }
+#endif
 
 KIntNumInput::KIntNumInput(QWidget *parent)
     : KNumInput(parent)
@@ -463,11 +470,13 @@ void KIntNumInput::setRange(int lower, int upper, int singleStep)
     setSliderEnabled(priv->slider);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KIntNumInput::setRange(int lower, int upper, int singleStep, bool slider)
 {
     setRange(lower, upper, singleStep);
     setSliderEnabled(slider);
 }
+#endif
 
 void KIntNumInput::setMinimum(int min)
 {
@@ -552,7 +561,7 @@ QSize KIntNumInput::minimumSizeHint() const
         h = qMax(h, priv->labelSize.height() + 2);
     }
 
-    w = priv->slider ? priv->slider->sizeHint().width() + 8 : 0;
+    w = priv->slider ? priv->slider->sizeHint().width() + KDialog::spacingHint() : 0;
     w += priv->column1Width + priv->column2Width;
 
     if (priv->labelAlignment & (Qt::AlignTop | Qt::AlignBottom)) {
@@ -590,9 +599,9 @@ void KIntNumInput::resizeEvent(QResizeEvent* e)
         priv->label->setGeometry(0, 0, w, d->intSpinBoxSize.height());
     }
 
-    if (qApp->layoutDirection()) {
+    if (qApp->layoutDirection() == Qt::RightToLeft) {
         d->intSpinBox->setGeometry(w, h, priv->slider ? priv->column2Width : qMax(priv->column2Width, e->size().width() - w), d->intSpinBoxSize.height());
-        w += priv->column2Width + 8;
+        w += priv->column2Width + KDialog::spacingHint();
 
         if (priv->slider) {
             priv->slider->setGeometry(w, h, e->size().width() - w, d->intSpinBoxSize.height() + KDialog::spacingHint());
@@ -701,6 +710,7 @@ KDoubleNumInput::KDoubleNumInput(double lower, double upper, double value, QWidg
     init(value, lower, upper, singleStep, precision);
 }
 
+#ifndef KDE_NO_DEPRECATED
 KDoubleNumInput::KDoubleNumInput(KNumInput *below,
                                  double lower, double upper, double value, QWidget *parent,
                                  double singleStep, int precision)
@@ -709,6 +719,7 @@ KDoubleNumInput::KDoubleNumInput(KNumInput *below,
 {
     init(value, lower, upper, singleStep, precision);
 }
+#endif
 
 KDoubleNumInput::~KDoubleNumInput()
 {
@@ -812,7 +823,7 @@ QSize KDoubleNumInput::minimumSizeHint() const
         h = qMax(h, priv->labelSize.height() + 2);
     }
 
-    w = priv->slider ? priv->slider->sizeHint().width() + 8 : 0;
+    w = priv->slider ? priv->slider->sizeHint().width() + KDialog::spacingHint() : 0;
     w += priv->column1Width + priv->column2Width;
 
     if (priv->labelAlignment & (Qt::AlignTop | Qt::AlignBottom)) {
@@ -838,7 +849,7 @@ void KDoubleNumInput::resizeEvent(QResizeEvent* e)
         priv->label->setGeometry(0, 0, w, d->editSize.height());
     }
 
-    if (qApp->layoutDirection()) {
+    if (qApp->layoutDirection() == Qt::RightToLeft) {
         d->spin->setGeometry(w, h, priv->slider ? priv->column2Width
                              : e->size().width() - w, d->editSize.height());
         w += priv->column2Width + KDialog::spacingHint();
