@@ -84,8 +84,12 @@ public:
     virtual void updateFromElement();
 
     virtual void layout();
+    virtual void calcWidth();
+    virtual void calcHeight();
     virtual void calcMinMaxWidth();
     virtual short baselinePosition( bool ) const;
+    virtual int calcContentWidth(int w) const;
+    virtual int calcContentHeight(int h) const;
 
     DOM::HTMLGenericFormElementImpl *element() const
     { return static_cast<DOM::HTMLGenericFormElementImpl*>(RenderObject::element()); }
@@ -96,15 +100,14 @@ public:
 protected:
     virtual bool isRenderButton() const { return false; }
     virtual bool isEditable() const { return false; }
-    Qt::Alignment textAlignment() const;
-
     virtual void setPadding();
-    KdeUiProxyStyle* getProxyStyle();
+    virtual void paintOneBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height);
 
-//     QPoint m_mousePos;
-//     int m_state;
+    Qt::Alignment textAlignment() const;
+    KdeUiProxyStyle* getProxyStyle();
     KdeUiProxyStyle *m_proxyStyle;
     bool m_exposeInternalPadding;
+    bool m_isOxygenStyle;
 };
 
 // -------------------------------------------------------------------------
@@ -304,7 +307,6 @@ public:
 
 protected:
     virtual bool event( QEvent *e );
-    virtual void paintEvent( QPaintEvent *pe );
     virtual void mouseMoveEvent(QMouseEvent *e);
     virtual void contextMenuEvent(QContextMenuEvent *e);
 private Q_SLOTS:
