@@ -83,9 +83,10 @@ bool StyleBoxData::operator==(const StyleBoxData& o) const
 	    max_width == o.max_width &&
 	    min_height == o.min_height &&
 	    max_height == o.max_height &&
-            box_sizing == o.box_sizing &&
+	    box_sizing == o.box_sizing &&
+	    vertical_align == o.vertical_align &&
 	    z_index == o.z_index &&
-            z_auto == o.z_auto;
+	    z_auto == o.z_auto;
 }
 
 StyleVisualData::StyleVisualData()
@@ -106,6 +107,8 @@ StyleVisualData::StyleVisualData(const StyleVisualData& o )
 
 BackgroundLayer::BackgroundLayer()
 :m_image(RenderStyle::initialBackgroundImage()),
+ m_xPosition(RenderStyle::initialBackgroundXPosition()),
+ m_yPosition(RenderStyle::initialBackgroundYPosition()),
  m_bgAttachment(RenderStyle::initialBackgroundAttachment()),
  m_bgClip(RenderStyle::initialBackgroundClip()),
  m_bgOrigin(RenderStyle::initialBackgroundOrigin()),
@@ -171,8 +174,8 @@ BackgroundLayer& BackgroundLayer::operator=(const BackgroundLayer& o) {
 
 bool BackgroundLayer::operator==(const BackgroundLayer& o) const {
     return m_image == o.m_image && m_xPosition == o.m_xPosition && m_yPosition == o.m_yPosition &&
-           m_bgAttachment == o.m_bgAttachment && m_bgClip == o.m_bgClip && m_bgOrigin == o.m_bgOrigin && m_bgRepeat == o.m_bgRepeat &&
-           m_backgroundSize.width == o.m_backgroundSize.width && m_backgroundSize.height == o.m_backgroundSize.height &&
+           m_bgAttachment == o.m_bgAttachment && m_bgClip == o.m_bgClip && m_bgOrigin == o.m_bgOrigin &&
+           m_bgRepeat == o.m_bgRepeat && m_backgroundSize == o.m_backgroundSize &&
            m_imageSet == o.m_imageSet && m_attachmentSet == o.m_attachmentSet && m_repeatSet == o.m_repeatSet &&
            m_xPosSet == o.m_xPosSet && m_yPosSet == o.m_yPosSet && m_backgroundSizeSet == o.m_backgroundSizeSet &&
            ((m_next && o.m_next) ? *m_next == *o.m_next : m_next == o.m_next);
@@ -181,88 +184,78 @@ bool BackgroundLayer::operator==(const BackgroundLayer& o) const {
 void BackgroundLayer::fillUnsetProperties()
 {
     BackgroundLayer* curr;
-    for (curr = this; curr && curr->isBackgroundImageSet(); curr = curr->next());
+
+    for (curr = this; curr && curr->isBackgroundXPositionSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_image = pattern->m_image;
+            curr->setBackgroundXPosition(pattern->backgroundXPosition());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundXPositionSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundYPositionSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_xPosition = pattern->m_xPosition;
+            curr->setBackgroundYPosition(pattern->backgroundYPosition());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundYPositionSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundAttachmentSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_yPosition = pattern->m_yPosition;
+            curr->setBackgroundAttachment(pattern->backgroundAttachment());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundAttachmentSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundClipSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_bgAttachment = pattern->m_bgAttachment;
+            curr->setBackgroundClip(pattern->backgroundClip());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundClipSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundOriginSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_bgClip = pattern->m_bgClip;
+            curr->setBackgroundOrigin(pattern->backgroundOrigin());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundOriginSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundRepeatSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_bgOrigin = pattern->m_bgOrigin;
+            curr->setBackgroundRepeat(pattern->backgroundRepeat());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
         }
     }
 
-    for (curr = this; curr && curr->isBackgroundRepeatSet(); curr = curr->next());
+    for (curr = this; curr && curr->isBackgroundSizeSet(); curr = curr->next()) {};
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
         for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_bgRepeat = pattern->m_bgRepeat;
-            pattern = pattern->next();
-            if (pattern == curr || !pattern)
-                pattern = this;
-        }
-    }
-
-    for (curr = this; curr && curr->isBackgroundSizeSet(); curr = curr->next());
-    if (curr && curr != this) {
-        // We need to fill in the remaining values with the pattern specified.
-        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
-            curr->m_backgroundSize = pattern->m_backgroundSize;
+            curr->setBackgroundSize(pattern->backgroundSize());
             pattern = pattern->next();
             if (pattern == curr || !pattern)
                 pattern = this;
@@ -275,11 +268,7 @@ void BackgroundLayer::cullEmptyLayers()
     BackgroundLayer *next;
     for (BackgroundLayer *p = this; p; p = next) {
         next = p->m_next;
-        if (next && !next->isBackgroundImageSet() &&
-            !next->isBackgroundXPositionSet() && !next->isBackgroundYPositionSet() &&
-            !next->isBackgroundAttachmentSet() && !next->isBackgroundClipSet() &&
-            !next->isBackgroundOriginSet() && !next->isBackgroundRepeatSet() &&
-            !next->isBackgroundSizeSet()) {
+        if (next && !next->isBackgroundImageSet()) {
             delete next;
             p->m_next = 0;
             break;
@@ -485,9 +474,9 @@ bool StyleCSS3InheritedData::operator==(const StyleCSS3InheritedData& o) const
 
 bool StyleCSS3InheritedData::shadowDataEquivalent(const StyleCSS3InheritedData& o) const
 {
-    if (!textShadow && o.textShadow || textShadow && !o.textShadow)
+    if ((!textShadow && o.textShadow) || (textShadow && !o.textShadow))
         return false;
-    if (textShadow && o.textShadow && (*textShadow != *o.textShadow))
+    if ((textShadow && o.textShadow) && (*textShadow != *o.textShadow))
         return false;
     return true;
 }
@@ -615,6 +604,28 @@ void RenderStyle::inheritFrom(const RenderStyle* inheritParent)
         setWhiteSpace(PRE_LINE);
 }
 
+void RenderStyle::compactWith(const RenderStyle* similarStyle)
+{
+    if (this == similarStyle) return;
+
+    if (box.get() != similarStyle->box.get() && box == similarStyle->box)
+        box = similarStyle->box;
+    if (visual.get() != similarStyle->visual.get() && visual == similarStyle->visual)
+        visual = similarStyle->visual;
+    if (background.get() != similarStyle->background.get() && background == similarStyle->background)
+        background = similarStyle->background;
+    if (surround.get() != similarStyle->surround.get() && surround == similarStyle->surround)
+        surround = similarStyle->surround;
+    if (generated.get() != similarStyle->generated.get() && generated == similarStyle->generated)
+        generated = similarStyle->generated;
+    if (css3NonInheritedData.get() != similarStyle->css3NonInheritedData.get() && css3NonInheritedData == similarStyle->css3NonInheritedData)
+        css3NonInheritedData = similarStyle->css3NonInheritedData;
+    if (css3InheritedData.get() != similarStyle->css3InheritedData.get() && css3InheritedData == similarStyle->css3InheritedData)
+        css3InheritedData = similarStyle->css3InheritedData;
+    if (inherited.get() != similarStyle->inherited.get() && inherited == similarStyle->inherited)
+        inherited = similarStyle->inherited;
+}
+
 RenderStyle::~RenderStyle()
 {
     RenderStyle *ps = pseudoStyle;
@@ -869,8 +880,8 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
         assert( other->position() != PSTATIC );                      // this style is positioned or relatively positioned
         if ( surround->hasSamePBMData(*other->surround.get()) &&    // padding/border/margin are identical
              (other->position() == PRELATIVE ||
-               !(other->left().isAuto() && other->right().isAuto()) &&  // X isn't static
-               !(other->top().isAuto() && other->bottom().isAuto()) ))   // neither is Y
+               (!(other->left().isAuto() && other->right().isAuto()) &&  // X isn't static
+                !(other->top().isAuto() && other->bottom().isAuto())) ))   // neither is Y
            // therefore only the offset is different
            return Position;
         return Layout;
@@ -927,8 +938,9 @@ void RenderStyle::setPaletteColor(QPalette::ColorGroup g, QPalette::ColorRole r,
 
 void RenderStyle::adjustBackgroundLayers()
 {
+    // https://www.w3.org/TR/css3-background/#layering
     if (backgroundLayers()->next()) {
-        // First we cull out layers that have no properties set.
+        // First we cull out backgroundLayers that have no image property set.
         accessBackgroundLayers()->cullEmptyLayers();
 
         // Next we repeat patterns into layers that don't have some properties set.
