@@ -208,6 +208,14 @@ struct CollapsedBorderValue
 class BorderData : public Shared<BorderData>
 {
 public:
+    BorderData() : Shared< khtml::BorderData >() {};
+    BorderData(const khtml::BorderData& other) : Shared<khtml::BorderData>() {
+        this->left = other.left;
+        this->right = other.right;
+        this->top = other.top;
+        this->bottom = other.bottom;
+    }
+
     BorderValue left;
     BorderValue right;
     BorderValue top;
@@ -307,8 +315,8 @@ public:
 
     EBoxSizing box_sizing;
 
-    signed int z_index :31;
-    bool z_auto        : 1;
+    int z_index;
+    bool z_auto;
 };
 
 //------------------------------------------------
@@ -345,7 +353,7 @@ public:
     StyleVisualData(const StyleVisualData& o );
 
     bool operator==( const StyleVisualData &o ) const {
-	return ( clip == o.clip &&
+	return ( clip == o.clip && textDecoration == o.textDecoration &&
 		 palette == o.palette );
     }
     bool operator!=( const StyleVisualData &o ) const {
@@ -376,9 +384,21 @@ enum EBackgroundAttachment {
     BGASCROLL, BGAFIXED, BGALOCAL
 };
 
-struct LengthSize {
+struct BGSize {
+    EBackgroundSizeType type : 2;
     Length width;
     Length height;
+
+    BGSize() :
+        type(BGSLENGTH), width(Length()), height(Length()) {}
+    BGSize(EBackgroundSizeType t) :   // Use to set BGSCONTAIN|BGSCOVER
+        type(t), width(Length()), height(Length()) { Q_ASSERT(t != BGSLENGTH); }
+    BGSize(Length w, Length h) :      // Use to set lenghts
+        type(BGSLENGTH), width(w), height(h) {}
+
+    bool operator==(const BGSize& o) const {
+        return type == o.type && width == o.width && height == o.height;
+    }
 };
 
 struct BackgroundLayer {
@@ -393,8 +413,7 @@ public:
     EBackgroundBox backgroundClip() const { return KDE_CAST_BF_ENUM(EBackgroundBox, m_bgClip); }
     EBackgroundBox backgroundOrigin() const { return KDE_CAST_BF_ENUM(EBackgroundBox, m_bgOrigin); }
     EBackgroundRepeat backgroundRepeat() const { return KDE_CAST_BF_ENUM(EBackgroundRepeat, m_bgRepeat); }
-    LengthSize backgroundSize() const { return m_backgroundSize; }
-    EBackgroundSizeType backgroundSizeType() const { return KDE_CAST_BF_ENUM(EBackgroundSizeType, m_bgSizeType); }
+    BGSize backgroundSize() const { return m_backgroundSize; }
 
     BackgroundLayer* next() const { return m_next; }
     BackgroundLayer* next() { return m_next; }
@@ -415,8 +434,7 @@ public:
     void setBackgroundClip(EBackgroundBox b) { m_bgClip = b; m_clipSet = true; }
     void setBackgroundOrigin(EBackgroundBox b) { m_bgOrigin = b; m_originSet = true; }
     void setBackgroundRepeat(EBackgroundRepeat r) { m_bgRepeat = r; m_repeatSet = true; }
-    void setBackgroundSize(const LengthSize& b) { m_backgroundSize = b; m_backgroundSizeSet = true; }
-    void setBackgroundSizeType(EBackgroundSizeType t) { m_bgSizeType = t; m_backgroundSizeSet = true; }
+    void setBackgroundSize(const BGSize& b) { m_backgroundSize = b; m_backgroundSizeSet = true; }
 
     void clearBackgroundImage() { m_imageSet = false; }
     void clearBackgroundXPosition() { m_xPosSet = false; }
@@ -462,9 +480,8 @@ public:
     KDE_BF_ENUM(EBackgroundBox) m_bgClip : 2;
     KDE_BF_ENUM(EBackgroundBox) m_bgOrigin : 2;
     KDE_BF_ENUM(EBackgroundRepeat) m_bgRepeat : 2;
-    KDE_BF_ENUM(EBackgroundSizeType) m_bgSizeType : 2;
 
-    LengthSize m_backgroundSize;
+    BGSize m_backgroundSize;
 
     bool m_imageSet : 1;
     bool m_attachmentSet : 1;
@@ -597,7 +614,13 @@ struct BorderRadii
 class BorderRadiusData : public Shared<BorderRadiusData>
 {
 public:
-    // default default & copy ctors are fine here.
+    BorderRadiusData() : Shared<BorderRadiusData>() {};
+    BorderRadiusData(const BorderRadiusData &other) : Shared<BorderRadiusData>(),
+        topRight(other.topRight),
+        bottomRight(other.bottomRight),
+        bottomLeft(other.bottomLeft),
+        topLeft(other.topLeft)
+    {}
 
     bool operator==(const BorderRadiusData& o) const;
     bool operator!=(const BorderRadiusData& o) const {
@@ -983,6 +1006,7 @@ public:
     ~RenderStyle();
 
     void inheritFrom(const RenderStyle* inheritParent);
+    void compactWith(const RenderStyle* similarStyle);
 
     PseudoId styleType() const { return KDE_CAST_BF_ENUM(PseudoId, noninherited_flags.f._styleType); }
     void setStyleType(PseudoId pi) { noninherited_flags.f._styleType = pi; }
@@ -1320,10 +1344,10 @@ public:
     bool flowAroundFloats() const { return  noninherited_flags.f._flowAroundFloats; }
     void setFlowAroundFloats(bool b=true) {  noninherited_flags.f._flowAroundFloats = b; }
 
-    int zIndex() const { return box->z_auto? 0 : box->z_index; }
+    int zIndex() const { return box->z_index; }
     void setZIndex(int v) { SET_VAR(box,z_auto,false ); SET_VAR(box, z_index, v); }
     bool hasAutoZIndex() const { return box->z_auto; }
-    void setHasAutoZIndex() { SET_VAR(box, z_auto, true ); }
+    void setHasAutoZIndex() { SET_VAR(box, z_auto, true ); SET_VAR(box, z_index, 0); }
 
     void setWidows(short w) { SET_VAR(inherited, widows, w); }
     void setOrphans(short o) { SET_VAR(inherited, orphans, o); }
@@ -1424,8 +1448,7 @@ public:
     static EBackgroundBox initialBackgroundClip() { return BGBORDER; }
     static EBackgroundBox initialBackgroundOrigin() { return BGPADDING; }
     static EBackgroundRepeat initialBackgroundRepeat() { return REPEAT; }
-    static LengthSize initialBackgroundSize() { return LengthSize(); }
-    static EBackgroundSizeType initialBackgroundSizeType() { return BGSLENGTH; }
+    static BGSize initialBackgroundSize() { return BGSize(); }
     static bool initialBorderCollapse() { return false; }
     static EBorderStyle initialBorderStyle() { return BNONE; }
     static ECaptionSide initialCaptionSide() { return CAPTOP; }
@@ -1449,8 +1472,8 @@ public:
     static ETextTransform initialTextTransform() { return TTNONE; }
     static EVisibility initialVisibility() { return VISIBLE; }
     static EWhiteSpace initialWhiteSpace() { return NORMAL; }
-    static Length initialBackgroundXPosition() { return Length(); }
-    static Length initialBackgroundYPosition() { return Length(); }
+    static Length initialBackgroundXPosition() { return Length(0.0, Percent); }
+    static Length initialBackgroundYPosition() { return Length(0.0, Percent); }
     static short initialBorderHorizontalSpacing() { return 0; }
     static short initialBorderVerticalSpacing() { return 0; }
     static ECursor initialCursor() { return CURSOR_AUTO; }
