@@ -30,11 +30,13 @@
 #include "xml/dom2_viewsimpl.h"
 #include "misc/idstring.h"
 #include <QDateTime>
+#include <QWeakPointer>
 
 #undef FOCUS_EVENT //for win32
 
 class QMouseEvent;
 class QKeyEvent;
+class KHTMLPart;
 
 namespace DOM {
 
@@ -164,32 +166,35 @@ public:
         DOMNODEINSERTEDINTODOCUMENT_EVENT,
         DOMATTRMODIFIED_EVENT,
         DOMCHARACTERDATAMODIFIED_EVENT,
-       // HTML events
-       LOAD_EVENT,
-       UNLOAD_EVENT,
-       ABORT_EVENT,
-       ERROR_EVENT,
-       SELECT_EVENT,
-       CHANGE_EVENT,
-       SUBMIT_EVENT,
-       RESET_EVENT,
-       FOCUS_EVENT,
-       BLUR_EVENT,
-       RESIZE_EVENT,
-       SCROLL_EVENT,
-        // keyboard events
-       KEYDOWN_EVENT,
-       KEYUP_EVENT,
-       KEYPRESS_EVENT, //Mostly corresponds to DOM3 textInput event.
-       // khtml events (not part of DOM)
-       KHTML_ECMA_DBLCLICK_EVENT, // for html ondblclick
-       KHTML_ECMA_CLICK_EVENT, // for html onclick
-       KHTML_DRAGDROP_EVENT,
-       KHTML_MOVE_EVENT,
-       KHTML_MOUSEWHEEL_EVENT,
-       KHTML_CONTENTLOADED_EVENT,
+        // HTML events
+        LOAD_EVENT,
+        UNLOAD_EVENT,
+        ABORT_EVENT,
+        ERROR_EVENT,
+        SELECT_EVENT,
+        CHANGE_EVENT,
+        SUBMIT_EVENT,
+        RESET_EVENT,
+        FOCUS_EVENT,
+        BLUR_EVENT,
+        RESIZE_EVENT,
+        SCROLL_EVENT,
+        HASHCHANGE_EVENT,
+            // keyboard events
+        KEYDOWN_EVENT,
+        KEYUP_EVENT,
+        KEYPRESS_EVENT, //Mostly corresponds to DOM3 textInput event.
+        // khtml events (not part of DOM)
+        KHTML_ECMA_DBLCLICK_EVENT, // for html ondblclick
+        KHTML_ECMA_CLICK_EVENT, // for html onclick
+        KHTML_DRAGDROP_EVENT,
+        KHTML_MOVE_EVENT,
+        KHTML_MOUSEWHEEL_EVENT,
+        KHTML_CONTENTLOADED_EVENT,
         // XMLHttpRequest events
-        KHTML_READYSTATECHANGE_EVENT
+        KHTML_READYSTATECHANGE_EVENT,
+        // HTML5 events
+        MESSAGE_EVENT
     };
 
     EventImpl();
@@ -219,6 +224,8 @@ public:
     virtual bool isMutationEvent() const;
     virtual bool isTextInputEvent() const;
     virtual bool isKeyboardEvent() const;
+    virtual bool isMessageEvent() const;
+    virtual bool isHashChangeEvent() const;
     bool isKeyRelatedEvent() const { return isTextInputEvent() || isKeyboardEvent(); }
 
     bool propagationStopped() const { return m_propagationStopped; }
@@ -575,5 +582,59 @@ protected:
     unsigned short m_attrChange;
 };
 
+class MessageEventImpl : public EventImpl {
+public:
+    enum DataType {
+        JS_VALUE
+    };
+
+    class Data : public khtml::Shared<Data> {
+    public:
+        virtual DataType messageDataType() const = 0;
+        virtual ~Data() {}
+    };
+
+    RefPtr<Data> data() const { return m_data; }
+    DOMString  origin() const { return m_origin; }
+    KHTMLPart* source() const { return m_source.data(); }
+    DOMString  lastEventId() const { return m_lastEventId; }
+
+    MessageEventImpl();
+    
+    void initMessageEvent(const DOMString &eventTypeArg,
+                          bool  canBubbleArg,
+                          bool  cancelableArg,
+                          const RefPtr<Data>& dataArg,
+                          const DOMString& originArg,
+                          const DOMString& lastEventIdArg,
+                          KHTMLPart* sourceArg); // no message ports yet.
+    virtual bool isMessageEvent() const;
+private:
+    RefPtr<Data> m_data;
+    DOMString    m_origin;
+    DOMString    m_lastEventId;
+    QWeakPointer<KHTMLPart>  m_source;
+};
+
+
+class HashChangeEventImpl : public EventImpl {
+public:
+    const DOMString &oldUrl() const { return m_oldUrl; }
+    const DOMString &newUrl() const { return m_newUrl; }
+
+    HashChangeEventImpl();
+
+    void initHashChangeEvent(const DOMString &eventTypeArg,
+                          bool  canBubbleArg,
+                          bool  cancelableArg,
+                          const DOMString &oldUrl,
+                          const DOMString &newUrl
+                         );
+    virtual bool isHashChangeEvent() const;
+private:
+    DOMString    m_oldUrl;
+    DOMString    m_newUrl;
+};
+
 } //namespace
 #endif
